// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FRAME_BINARY_OPERATION_H
#define FRAME_BINARY_OPERATION_H

#include "Node.h"
#include "FrameOperation.h"

/**Class for nodes that operate on frames (typically audio) using a single input and a single output*/
class FrameBinaryOperation : public FrameOperation {
protected:
   /**Length of input frames*/
   int input1Length;

   /** inputID #1*/
   int input1ID;

   /**Length of input frames*/
   int input2Length;

   /** inputID #2*/
   int input2ID;

   /**input1Buffer*/
   ObjectRef input1Buffer;

   /**input2Buffer*/
   ObjectRef input2Buffer;

public:
   /**Constructor, takes the name of the node and a set of parameters*/
   FrameBinaryOperation(string nodeName, const ParameterSet &params);

   /**Class specific initialization routine.
      Each class will call its subclass specificInitialize() method*/
   virtual void specificInitialize();

   /**Class reset routine.
      Each class will call its superclass reset() method*/
   virtual void reset();

   /**Ask for the node's output which ID (number) is output_id 
      and for the 'count' iteration */
   virtual ObjectRef getOutput(int output_id, int count); 

protected:
   /**Default constructor, should not be used*/
   FrameBinaryOperation() {throw new GeneralException("FrameBinaryOperation copy constructor should not be called",__FILE__,__LINE__);}

   /**Computes the output for one frame*/
   virtual void computeFrame(ObjectRef input1, ObjectRef input2, int count) = 0;

};

#endif
