// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "FrameBinaryOperation.h"
#include "FrameOperation.h"
#include "Vector.h"
#include "Buffer.h"
#include "multithread.h"
#include "RotatingBuffer.h"

FrameBinaryOperation::FrameBinaryOperation(string nodeName, const ParameterSet &params) 
   : FrameOperation(nodeName, params) 
{
   input1ID = addInput("INPUT1");
   if (parameters.exist("LOOKAHEAD1"))
      inputsCache[input1ID].lookAhead = dereference_cast<int> (parameters.get("LOOKAHEAD1"));
   if (parameters.exist("LOOKBACK1"))
      inputsCache[input1ID].lookBack = dereference_cast<int> (parameters.get("LOOKBACK1"));
   if (parameters.exist("INPUT1LENGTH"))
      input1Length = dereference_cast<int> (parameters.get("INPUT1LENGTH"));
   else input1Length = dereference_cast<int> (parameters.get("LENGTH"));

   input2ID = addInput("INPUT2");
   if (parameters.exist("LOOKAHEAD2"))
      inputsCache[input2ID].lookAhead = dereference_cast<int> (parameters.get("LOOKAHEAD2"));
   if (parameters.exist("LOOKBACK2"))
      inputsCache[input2ID].lookBack = dereference_cast<int> (parameters.get("LOOKBACK2"));
   if (parameters.exist("INPUT2LENGTH"))
      input2Length = dereference_cast<int> (parameters.get("INPUT2LENGTH"));
   else input2Length = dereference_cast<int> (parameters.get("LENGTH"));
}

void FrameBinaryOperation::specificInitialize()
{
   //cerr << "FrameBinaryOperation initialize...\n";
   this->FrameOperation::specificInitialize();
   input1Buffer = ObjectRef(new RotatingBuffer (inputsCache[input1ID].lookAhead+inputsCache[input1ID].lookBack+1));
   input2Buffer = ObjectRef(new RotatingBuffer (inputsCache[input2ID].lookAhead+inputsCache[input2ID].lookBack+1));
}

void FrameBinaryOperation::reset()
{
   this->FrameOperation::reset();
   input1Buffer = ObjectRef(new RotatingBuffer (inputsCache[input1ID].lookAhead+inputsCache[input1ID].lookBack+1));
   input2Buffer = ObjectRef(new RotatingBuffer (inputsCache[input2ID].lookAhead+inputsCache[input2ID].lookBack+1));
}

ObjectRef FrameBinaryOperation::getOutput(int output_id, int count)
{
   //cerr << "FrameBinaryOperation::getOutput for " << name << " count = " << count << endl;

   lock();
   try {
      
      /*NodeInput input = inputs[inputID];
        for (int i = count-inputLookBack; i <= count+inputLookAhead ; i++)
        {
        if (!inputBuffer[i]->isValid())
        inputBuffer[i] = input.node->getOutput(input.outputID, i);
        }*/
      
      Buffer &out = object_cast<Buffer> (output);
      Buffer &in1 = object_cast<Buffer> (input1Buffer);
      Buffer &in2 = object_cast<Buffer> (input2Buffer);

      //cerr << "FrameBinaryOperation::getOutput for " << name << " count: " << count << endl;
      for (int outputCount = processCount+1 ; outputCount <= count; outputCount++)
      {
         processCount = outputCount;
         NodeInput input1 = inputs[input1ID];
         NodeInput input2 = inputs[input2ID];
         
         bool invalid = false;
         if (outputCount-inputsCache[input1ID].lookBack < 0 || outputCount-inputsCache[input2ID].lookBack < 0)
         {
            out[outputCount]->status = Object::before_beginning;
            continue;
         }
         //cerr << "getting inputs for " << name << " count: " << outputCount << endl;
         for (int input1Count = outputCount-inputsCache[input1ID].lookBack; input1Count <= outputCount + inputsCache[input1ID].lookAhead ; input1Count++)
         {
            if (in1[input1Count]->status != Object::valid)
            {
               in1[input1Count] = input1.node->getOutput(input1.outputID, input1Count);
               if (in1[input1Count]->status != Object::valid)
               {
                  out[outputCount]->status = in1[input1Count]->status;
                  invalid = true; 
                  break;
               }
            }
         }
         for (int input2Count = outputCount-inputsCache[input2ID].lookBack; input2Count <= outputCount + inputsCache[input2ID].lookAhead ; input2Count++)
         {
            if (in2[input2Count]->status != Object::valid)
            {
               in2[input2Count] = input2.node->getOutput(input2.outputID, input2Count);
               if (in2[input2Count]->status != Object::valid)
               {
                  out[outputCount]->status = in2[input2Count]->status;
                  invalid = true; 
                  break;
               }
            }
         }

         //cerr << "input invalid? " << invalid<< endl;
         
         if (!invalid)
         {
            //cerr << "computing\n";
            this->computeFrame(input1Buffer, input2Buffer, outputCount);
            out[outputCount]->status = Object::valid;
         }
      }
      //unlock();
      //cerr << "leaving FrameBinaryOperation::getOutput for " << name << " count: " << count << endl;
      //cerr << "returning status " << out[count]->status << endl;
      return unlock_and_return(out[count]);
   } catch (BaseException *e)
   {
      //e->print();
      unlock();
      throw e->add(new NodeException (this, "Exception in FrameBinaryOperation::getOutput", __FILE__, __LINE__));
   }


}
