// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "FrameUnaryOperation.h"
#include "FrameOperation.h"
#include "Vector.h"
#include "multithread.h"
#include "Buffer.h"
#include "RotatingBuffer.h"

FrameUnaryOperation::FrameUnaryOperation(string nodeName, const ParameterSet &params) 
   : FrameOperation(nodeName, params) 
{
   inputID = addInput("INPUT");
   if (parameters.exist("LOOKAHEAD"))
      inputsCache[inputID].lookAhead = dereference_cast<int> (parameters.get("LOOKAHEAD"));
   if (parameters.exist("LOOKBACK"))
      inputsCache[inputID].lookBack = dereference_cast<int> (parameters.get("LOOKBACK"));
   if (parameters.exist("INPUTLENGTH"))
      inputLength = dereference_cast<int> (parameters.get("INPUTLENGTH"));
   else inputLength = dereference_cast<int> (parameters.get("LENGTH"));
}

void FrameUnaryOperation::specificInitialize()
{
   //cerr << "FrameUnaryOperation initialize...\n";
   this->FrameOperation::specificInitialize();
   
   inputBuffer = ObjectRef(new RotatingBuffer (inputsCache[inputID].lookAhead+inputsCache[inputID].lookBack+1));

}

void FrameUnaryOperation::reset()
{
   this->FrameOperation::reset();

   inputBuffer = ObjectRef(new RotatingBuffer (inputsCache[inputID].lookAhead+inputsCache[inputID].lookBack+1));
}

void FrameUnaryOperation::calculate(int output_id, int count, Buffer &out)
{
   lock();
   try {      
      Buffer &in = object_cast<Buffer> (inputBuffer);
      
      NodeInput input = inputs[inputID];
      
      if (count-inputsCache[inputID].lookBack < 0)
      {
         out[count]->status = Object::before_beginning;
         return;
      }
      
      for (int inputCount = count-inputsCache[inputID].lookBack; inputCount <= count + inputsCache[inputID].lookAhead ; inputCount++)
      {
         if (in[inputCount]->status != Object::valid)
         {
            in[inputCount] = input.node->getOutput(input.outputID, inputCount);
            if (in[inputCount]->status != Object::valid)
            {
               out[count]->status = in[inputCount]->status;
               return;
            }
         }
      }
      
      this->computeFrame(inputBuffer, count);
      out[count]->status = Object::valid;
      
   } catch (BaseException *e)
   {
      e->print();
      unlock();
      throw new NodeException (this, "Exception in FrameUnaryOperation::getOutput", __FILE__, __LINE__);
   }
}
