// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#ifndef NetworkBuilder_h
#define NetworkBuilder_h

/*
** Includes
*/
#include <string>
#include "Network.h"
#include "FP_network_info_container.h"
#include "NetworkBuilderException.h"
#include <list>

/** The NetworkBuilder class.
 ** A NetworkBuilder object has the functionality of taking the ouput of a file parser
 ** and building a Network object out of it.  The file parser used must give its output 
 ** as a map of FP_network_info_container objects, each indexed by network name.
 ** The NetworkBuilder object will build a Network object and return a pointer to it,
 ** so it is the calling application's responsibility to destroy the Network afterwards.
 ** @author Andre Charbonneau
 **/
class NetworkBuilder
{
 private:

  bool m_verbose;

  list<string> create_building_list(const map<string,FP_network_info_container>& net_info_map, const string& net_name) const throw(NetworkBuilderException);

  map<string,FP_network_info_container> crunch_subnets(const map<string,FP_network_info_container>& net_info_map, const string& net_name) throw(NetworkBuilderException);

  string find_unique_net_name(const map<string,FP_network_info_container>& net_info_map, const string& prefix);

 public:
  /// Default constructor
  NetworkBuilder();

  /// Destructor
  virtual ~NetworkBuilder();

  /** This method is used to build a data-flow network.
   ** @param net_info_map The information about the network to build.
   ** @param net_name The name of the network to build.
   ** @return A pointer to the new Network object.
   ** @exception NetworkBuilderException
   **/
  Network* build(const map<string,FP_network_info_container>& net_info_map, 
		 const string& net_name, 
		 ParameterSet* extern_ps = NULL) throw(NetworkBuilderException);

  /** Sets the verbose flag.
   **
   */
  void set_verbose(const bool b);

  /** Gets the value of the verbose flag.
   ** @return true if verbose mode is on, else returns false.
   */
  bool verbose() const;

  
};

#endif
