// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef NetworkBuilderException_h
#define NetworkBuilderException_h

/*
** Includes
*/
#include "Exception.h"

/*
** Defines
*/
#define NB_NO_TYPE -1
#define NB_FATAL_ERROR 0
#define NB_NOT_FOUND_EXCEPTION 1
#define NB_NETWORK_EXCEPTION 2
#define NB_RECURSION_EXCEPTION 3
#define NB_NETWORK_PARSER_EXCEPTION 4
#define NB_EXTERNAL_PARAMETER_EXCEPTION 5
#define NB_ITERATOR_NET_EXCEPTION 6


/**
 ** The BuilderException class implements exception handling for everything that
 ** is relating to data-flow network building.
 */
class NetworkBuilderException : public BaseException
{
 private:
  /** The type of the exception. Valid types are:
   ** NB_NO_TYPE,NB_FATAL_ERROR
   */
  int m_type;

  /// The description of the exception.
  string m_description;


 public:
  /** Constructor
   ** @param type The type of the exception. Must be in {NB_NO_TYPE,NB_FATAL_ERROR,NB_NOT_FOUND_EXCEPTION,NB_NETWORK_EXCEPTION}
   ** @param description The description of the exception. (optional)
   */
  NetworkBuilderException(int type, const string& description = "");



  /** This method is used to print the exception to a stream, cerr by default.
   ** @param out The stream to print to. (default to cerr)
   ** @return void
   */
  virtual void print(ostream &out = cerr);


  /** Gets the description of the exception.
   ** @return The description as a string.
   */
  string get_description() const;


  /** Gets the type of the exception.
   ** @return The description's type as an integer.
   */
  int get_type() const;

};

#endif
