// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/*
** Includes
*/
#include "NetworkBuilderException.h"

NetworkBuilderException::NetworkBuilderException(int type, const string& description)
{
  m_type = type;
  m_description = description;
  return;
}

void NetworkBuilderException::print(ostream &out)
{
  string type_string = "";
  switch(m_type)
    {
    case NB_NO_TYPE:
      {
	type_string = "No exception type.";
	break;
      }
    case NB_FATAL_ERROR:
      {
	type_string = "Fatal error.";
	break;
      }
    case NB_NOT_FOUND_EXCEPTION:
      {
	type_string = "'Not found' exception.";
	break;
      }
    case NB_NETWORK_EXCEPTION:
      {
	type_string = "A exception derived from BaseException was caught while building the network.";
	break;
      }
    case NB_ITERATOR_NET_EXCEPTION:
      {
	type_string = "Iterator net related exception.";
	break;
      }
    default:
      {
	type_string = "";
	break;
      }
    }
  out << "NetworkBuilderException: " << type_string << ": " << this->get_description() << endl;

  return;
}

string NetworkBuilderException::get_description() const
{
  return m_description;
}

int NetworkBuilderException::get_type() const
{
  return m_type;
}
