#include <tree.h>
#include <parser.h>
#include "UIDocument.h"
#include "UINetwork.h"
//#include "UINodeMenu.h"
#include "Node.h"
#include "Network.h"
#include "ParameterSet.h"



map<string, SubnetInfo *> UIDocument::externalDocInfo;


UIDocument::UIDocument(string _name)
   : docName(_name)
   , untitled(true)
   , modified(false)
{
   //cerr << "created doc with name " << docName << endl;
   //gnome_mdi_child_set_name (GNOME_MDI_CHILD(this), (gchar *)name.c_str());
   //create();
}

UIDocument::~UIDocument()
{
   //cerr << "destroying UIDocument " << name << endl;
   for (int i=0;i<networks.size();i++)
      delete networks[i];
   //child.name = "destroyed document";
}


UINetwork *UIDocument::getNetworkNamed(const string &n)
{
   //cerr << "UIDocument::getNetworkNamed\n";
   //cerr << n << endl;
   //cerr << "size = " << networks.size() << endl;
   for (int i=0;i<networks.size();i++)
   {
      //cerr << "ptr = " << &(networks[i]) << endl;
      //cerr << networks[i]->getName() << " == " << n << endl;
      if (networks[i]->getName() == n)
         return networks[i];
   }
   //cerr << "net not found in doc\n";
   return NULL;
}

vector<string> UIDocument::getNetInputs(const string &netName)
{
   UINetwork *net = getNetworkNamed(netName);
   vector <string> inputs;
   if (net)
   {
      inputs = net->getTerminals(UINetTerminal::INPUT);
   } else if (preloadInfo.find(netName) != preloadInfo.end()) {
      inputs = preloadInfo[netName]->inputs;
   } else if (externalDocInfo.find(netName) != externalDocInfo.end()) {
      inputs = externalDocInfo[netName]->inputs;
   }
   return inputs;
}

vector<string> UIDocument::getNetOutputs(const string &netName)
{
   UINetwork *net = getNetworkNamed(netName);
   vector <string> outputs;
   if (net)
   {
      outputs = net->getTerminals(UINetTerminal::OUTPUT);
   } else if (preloadInfo.find(netName) != preloadInfo.end()) {
      outputs = preloadInfo[netName]->outputs;
   } else if (externalDocInfo.find(netName) != externalDocInfo.end()) {
      outputs = externalDocInfo[netName]->outputs;
   }
   return outputs;
}

vector<string> UIDocument::getNetParams(const string &netName)
{
   //cerr << "UIDocument::getNetParams for document: " << docName << "\n";
   UINetwork *net = getNetworkNamed(netName);
   //cerr << "got the network\n";
   vector <string> params;
   if (net)
   {
      //cerr << "net is already loaded...\n";
      net->insertNetParams(params);
   } else if (preloadInfo.find(netName) != preloadInfo.end()) {
      //cerr << "net is not loaded...\n";
      params = preloadInfo[netName]->params;
   } else if (externalDocInfo.find(netName) != externalDocInfo.end()) {
      //cerr << "net is not loaded...\n";
      params = externalDocInfo[netName]->params;
   }
   return params;
}

void UIDocument::loadNetInfo(xmlNodePtr net, map<string, SubnetInfo *> &infoMap, string netName)
{
   if (netName == "")
      netName = string((char *)xmlGetProp(net, (CHAR *)"name"));
   
   if (infoMap.find(netName) != infoMap.end())
   {
      cerr << "error: net already existed\n";
      return;
   }
   //cerr << "new subnet info with name: " << netName << "\n";
   SubnetInfo *info = new SubnetInfo;
   infoMap[netName] = info;

	 //cerr << "scan all nodes\n";
   xmlNodePtr node = net->childs;
   while (node != NULL)
   {
      if (string((char*)node->name) == "Node")
      {

	       
	 xmlNodePtr par = node->childs;
	 //cerr << "par = " << par << endl;
	 while (par)
	 {
	    if (string((char*)par->name) == "Parameter")
	    {
	       string paramName = string ((char *) xmlGetProp(par, (CHAR *)"value"));
	       string type = string ((char *) xmlGetProp(par, (CHAR *)"type"));
	       if (type == "subnet_param")
	       {
		  bool alreadyPresent = false;
		  for (int j=0;j<info->params.size();j++)
		     if (info->params[j] == paramName)
			alreadyPresent=true;
		  if (!alreadyPresent)
		     info->params.insert (info->params.end(), paramName);
	       }		  
	    }
	    par = par->next;
		  
	 }



	 //check for params
      }
      node = node->next;
   }

   //scan all net inputs/outputs
   node = net->childs;
   while (node != NULL)
   {
      if (string((char*)node->name) == "NetInput")
      {
	 string termName = string((char *)xmlGetProp(node, (CHAR *)"name"));
	 info->inputs.insert (info->inputs.end(), termName);
	       
      } else if (string((char*)node->name) == "NetOutput")
      {
	 string termName = string((char *)xmlGetProp(node, (CHAR *)"name"));
	 info->outputs.insert (info->outputs.end(), termName);
	       
      }
      node = node->next;
   }
 
}

void UIDocument::loadAllSubnetInfo(xmlNodePtr net)
{
   //cerr << "UIDocument::loadNetInfo\n";
   //cerr << "this = " << this << endl;
   while (net != NULL)
   {
      if (string((char*)net->name) == "Network")
      {
	 //cerr << "scanning a net\n";

	 loadNetInfo(net, preloadInfo);
	 
      }
      net = net->next;
   }
}

void UIDocument::load()
{
   string fullpath=path+docName;
   xmlDocPtr doc = xmlParseFile(fullpath.c_str());

   if (!doc || !doc->root || !doc->root->name)
   {
      cerr << "error loading\n";
      xmlFreeDoc (doc);
      return;
   }
   xmlNodePtr root=doc->root;

   loadAllSubnetInfo(root->childs);

   xmlNodePtr net = root->childs;
   //cerr << "parsing...\n";
   while (net != NULL)
   {
      //cerr << "start net\n";
      if (string((char*)net->name) == "Network")
         addNetwork (net);
      net = net->next;
   }


   vector<string> tmp = getNetParams("MAIN");
   //cerr << "Got " << tmp.size() << " params in GUIDocument::createParamDialog\n";
   params.resize(tmp.size());
   for (int i=0;i<tmp.size();i++)
      params[i].name=tmp[i];
   //cerr << "--\n";
   xmlNodePtr par = root->childs;
   //cerr << "par = " << par << endl;
   while (par)
   {
      if (string((char*)par->name) == "Parameter")
      {
         string name = string ((char *) xmlGetProp(par, (CHAR *)"name"));
         string type = string ((char *) xmlGetProp(par, (CHAR *)"type"));
         string value = string ((char *) xmlGetProp(par, (CHAR *)"value"));
         
         for (int i=0;i<params.size();i++)
	 {
	    if (params[i].name == name)
	    {
	       params[i].type = type;
	       params[i].value = value;
	       //insertLoadedParam(param, type, value);
	    }
	 }
         //cerr << "<param: " << name << ", " << type << ":" << value << ">\n";
      }
      par = par->next;
      
   }

   xmlFreeDoc(doc);

}

static vector<string> envList(char *envName)
{
   vector<string> list;
   char *strPath = getenv(envName);
   if (!strPath)
      return list;
   string path = strPath; 
   int start = 0;
   int pos = 0;
   while (pos < path.length())
   {
      if (path[pos] == ':')
      {
	 list.insert(list.end(), string(&(path[start]), &(path[pos])));
	 start = pos+1;
      }
      pos++;
   }
   if (pos)
      list.insert(list.end(), string(&(path[start]), &(path[pos])));

   //cerr << pathList << endl;
   return list;
}


#include <sys/types.h>

#include <dirent.h>

void UIDocument::loadExtDocInfo()
{
   vector<string> dirs = envList("VFLOW_PATH");
   for (int i=0;i<dirs.size();i++)
   {
      string path = dirs[i];
      vector<string> files;
      DIR *my_directory = opendir (path.c_str());
      if (!my_directory) 
	 continue;
      struct dirent *current_entry;
      for (current_entry = readdir(my_directory); 
	   current_entry != NULL; current_entry = readdir(my_directory)) 
      {
	 if (!strstr(current_entry->d_name, ".n"))
	 {
	    //cerr << current_entry->d_name << " is not a document\n";
	    continue;
	 }
	 string name = current_entry->d_name;
	 string fullname = path + "/" + name;
	 //cerr << "scanning file...\n";
	 xmlDocPtr doc = xmlParseFile(fullname.c_str());
	 
	 if (!doc || !doc->root || !doc->root->name)
	 {
	    cerr << "error loading\n";
	    xmlFreeDoc (doc);
	    continue;
	 }
	 xmlNodePtr root=doc->root;
	 xmlNodePtr net = root->childs;
	 
	 while (net != NULL)
	 {
	    //cerr << "scanning networks...\n";
	    if (string((char*)net->name) == "Network")
	    {
	       //cerr << "scanning a net\n";
	       string netName = string((char *)xmlGetProp(net, (CHAR *)"name"));
	       if (netName == "MAIN")
	       {
		  string basename = string(name.begin(), name.end()-2);
		  loadNetInfo(net, externalDocInfo, basename);
	       }
	    }
	    net = net->next;
	 }
	 xmlFreeDoc(doc);
      }
      closedir(my_directory);
   }
}

#include <dlfcn.h>

void UIDocument::scanDL()
{
   vector<string> dirs=envList("VFLOW_PATH");
   if (dirs.size() == 0)
   {
      cerr << "Did you forget to set the VFLOW_PATH environment to the directory where the libraries are installed?\n";
   }
   for (int i = 0; i<dirs.size();i++)
   {
      DIR *my_directory = opendir (dirs[i].c_str());
      if (!my_directory)
	 continue;
      struct dirent *current_entry;
      for (current_entry = readdir(my_directory); 
	   current_entry != NULL; current_entry = readdir(my_directory)) 
      {
	 if (!strstr(current_entry->d_name, ".so"))
	 {
	    //cerr << current_entry->d_name << " is not a shared library\n";
	    continue;
	 }
	 string fullname = dirs[i] + "/" + current_entry->d_name;
	 dlopen(fullname.c_str(), RTLD_LAZY);
      }
      
      closedir(my_directory);
   }
   //UIDocument::loadExtDocInfo(files);
}


UINetwork *UIDocument::newNetwork(UIDocument *_doc, const string &_name, UINetwork::Type type)
{
   //cerr << "UIDocument::newNetwork\n";
   return new UINetwork(_doc, _name, type);
}

UINetwork *UIDocument::newNetwork(UIDocument *_doc, xmlNodePtr _net)
{
   //cerr << "UIDocument::newNetwork\n";
   return new UINetwork(_doc, _net);
}


UINetwork *UIDocument::addNetwork(string name, UINetwork::Type type)
{
   //cerr << "UIDocument::addNetwork (type = " << typeid(this).name() << ")" << endl;
   
   //UINetwork *newNet = new GUINetwork(this, name, iter);
   UINetwork *newNet = newNetwork(this, name, type);
   for (int i=0;i<networks.size();i++)
   {
      networks[i]->newNetNotify("Subnet",name);
      newNet->newNetNotify("Subnet",networks[i]->getName());
   }
   networks.insert(networks.end(), newNet);
   modified = true;
   return newNet;
}

UINetwork *UIDocument::addNetwork(xmlNodePtr xmlNet)
{
   //cerr << "creating...\n";
   UINetwork *newNet = newNetwork(this, xmlNet);
   //cerr << "created\n";
   //cerr << "newNet = " << newNet << endl;
   //cerr << "network created in UIDocument::addNetwork\n";
   for (int i=0;i<networks.size();i++)
   {
      networks[i]->newNetNotify("Subnet",newNet->getName());
      newNet->newNetNotify("Subnet",networks[i]->getName());
   }
   //cerr << "newNet = " << newNet << endl;
   networks.insert(networks.end(), newNet);

   modified = true;
   return newNet;
}

void UIDocument::save()
{
   xmlDocPtr doc;
   doc = xmlNewDoc((CHAR *)"1.0");
   doc->root = xmlNewDocNode(doc, NULL, (CHAR *)"Document", NULL);
   for (int i=0;i<networks.size();i++)
   {
      networks[i]->saveXML(doc->root);
   }

   for (int i=0;i<params.size();i++)
   {
      xmlNodePtr tree;
      tree = xmlNewChild(doc->root, NULL, (CHAR *)"Parameter", NULL);
      xmlSetProp(tree, (CHAR *)"name", (CHAR *)params[i].name.c_str());
      xmlSetProp(tree, (CHAR *)"type", (CHAR *)params[i].type.c_str());
      xmlSetProp(tree, (CHAR *)"value", (CHAR *)params[i].value.c_str());
   }

   string fullname = path+docName;
   xmlSaveFile(fullname.c_str(), doc);

   xmlFreeDoc(doc);
}

void UIDocument::export2net()
{
   string netName = path+docName+"et";
   ofstream out(netName.c_str());;
   for (int i=0;i<networks.size();i++)
   {
      networks[i]->export2net(out);
   }
   
}

#include <sys/stat.h>
#include <unistd.h>

Network *UIDocument::buildExternal(const string &type, const string &_name, const ParameterSet &params)
{
   string filename = "";
   vector<string> pathlist = envList("VFLOW_PATH");
   for (int i=0;i<pathlist.size();i++)
   {
      string fullname = pathlist[i]+"/"+type+".n";
      struct stat buf;
      //cerr << "trying " << fullname << endl;
      if (stat(fullname.c_str() , &buf) == 0)
      {
	 //cerr << "found!\n";
	 filename = fullname;
	 break;
      }
   }
   if (filename == "")
   {
      cerr << "error loading: not found\n";
      return NULL;
   }
   UIDocument doc(filename);
   doc.load();
   Network *net = doc.getNetworkNamed("MAIN")->build(_name, params);
   //cerr << "net = " << net << endl;
   return net;
   //cerr << "UIDocument::build\n";
   //return getNetworkNamed("MAIN")->build(_name, params);
}

Network *UIDocument::build(const string &_name, const ParameterSet &params)
{
   //cerr << "UIDocument::build\n";
   return getNetworkNamed("MAIN")->build(_name, params);
}

//Run without a GUI
void UIDocument::run()
{
  try {
   ParameterSet params;
   //cerr << "building net...\n";
   Network *net = build("MAIN", params);
   //cerr << "initializing...\n";
   net->initialize();
   //cerr << "running (UIDocument)...\n";

   for (int i = 0; ;i++) {
     if (!net->hasOutput(i)) break;
     cout << *net->getOutput(i,0);
   }
  }
  catch (BaseException &e) {
    e.print();
  }
  catch (BaseException *e) {
    e->print();
  }

}

void UIDocument::run(ParameterSet &p)
{
   try {
     //cerr << "building net...\n";
     Network *net = build("MAIN", p);
     //cerr << "initializing...\n";
     net->initialize();
     //cerr << "running (UIDocument)...\n";
     for (int i = 0; ;i++) {
       if (!net->hasOutput(i)) break;
       cout << *net->getOutput(i,0);
     }
   } 
   catch (BaseException &e) {
     e.print();
   }
   catch (BaseException *e) {
     e->print();
   }
   
}
